@@ -369,6 +369,10 @@ webkit_pref_callback_accept_languages (GSettings  *settings,
 
   webkit_web_context_set_preferred_languages (web_context, (const char * const *)(void *)array->data);
 
+  /* Set preferred languages also for the default web context, used by the sync
+   * tab in the preferences dialog. It doesn't need spellchecking. */
+  webkit_web_context_set_preferred_languages (webkit_web_context_get_default (), (const char * const *)(void *)array->data);
+
   if (g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING)) {
     char **normalized = normalize_languages ((char **)(void *)array->data);
     webkit_web_context_set_spell_checking_languages (web_context, (const char * const *)normalized);
