@@ -34,7 +34,7 @@
 #include "ephy-profile-utils.h"
 #include "ephy-settings.h"
 #include "ephy-snapshot-service.h"
-#include "ephy-uri-tester.h"
+#include "ephy-uri-tester-shared.h"
 #include "ephy-view-source-handler.h"
 #include "ephy-web-app-utils.h"
 #include "ephy-web-extension-proxy.h"
@@ -43,10 +43,16 @@
 #include <gtk/gtk.h>
 #include <stdlib.h>
 
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+#include <httpseverywhere.h>
+#endif
+
 #define PAGE_SETUP_FILENAME "page-setup-gtk.ini"
 #define PRINT_SETTINGS_FILENAME "print-settings.ini"
 #define OVERVIEW_RELOAD_DELAY 500
 
+#define ADBLOCK_FILTER_UPDATE_FREQUENCY 24 * 60 * 60 /* In seconds */
+
 typedef struct {
   WebKitWebContext *web_context;
   EphyHistoryService *global_history_service;
@@ -61,9 +67,11 @@ typedef struct {
   EphyViewSourceHandler *source_handler;
   guint update_overview_timeout_id;
   guint hiding_overview_item;
-  EphyUriTester *uri_tester;
   GDBusServer *dbus_server;
   GList *web_extensions;
+
+  char *adblock_data_dir;
+  GCancellable *uri_tester_update_cancellable;
 } EphyEmbedShellPrivate;
 
 enum {
@@ -72,6 +80,7 @@ enum {
   PAGE_CREATED,
   ALLOW_TLS_CERTIFICATE,
   FORM_AUTH_DATA_SAVE_REQUESTED,
+  SENSITIVE_FORM_FOCUSED,
 
   LAST_SIGNAL
 };
@@ -95,6 +104,11 @@ ephy_embed_shell_dispose (GObject *object)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (EPHY_EMBED_SHELL (object));
 
+  if (priv->uri_tester_update_cancellable) {
+    g_cancellable_cancel (priv->uri_tester_update_cancellable);
+    g_clear_object (&priv->uri_tester_update_cancellable);
+  }
+
   if (priv->update_overview_timeout_id > 0) {
     g_source_remove (priv->update_overview_timeout_id);
     priv->update_overview_timeout_id = 0;
@@ -110,12 +124,21 @@ ephy_embed_shell_dispose (GObject *object)
   g_clear_object (&priv->downloads_manager);
   g_clear_object (&priv->hosts_manager);
   g_clear_object (&priv->web_context);
-  g_clear_object (&priv->uri_tester);
   g_clear_object (&priv->dbus_server);
 
   G_OBJECT_CLASS (ephy_embed_shell_parent_class)->dispose (object);
 }
 
+static void
+ephy_embed_shell_finalize (GObject *object)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (EPHY_EMBED_SHELL (object));
+
+  g_free (priv->adblock_data_dir);
+
+  G_OBJECT_CLASS (ephy_embed_shell_parent_class)->finalize (object);
+}
+
 static void
 web_extension_form_auth_data_message_received_cb (WebKitUserContentManager *manager,
                                                   WebKitJavascriptResult   *message,
@@ -138,6 +161,26 @@ web_extension_form_auth_data_message_received_cb (WebKitUserContentManager *mana
   g_variant_unref (variant);
 }
 
+static void
+web_extension_sensitive_form_focused_message_received_cb (WebKitUserContentManager *manager,
+                                                          WebKitJavascriptResult   *message,
+                                                          EphyEmbedShell           *shell)
+{
+  guint64 page_id;
+  gboolean insecure_action;
+  GVariant *variant;
+  char *message_str;
+
+  message_str = ephy_embed_utils_get_js_result_as_string (message);
+  variant = g_variant_parse (G_VARIANT_TYPE ("(tb)"), message_str, NULL, NULL, NULL);
+  g_free (message_str);
+
+  g_variant_get (variant, "(tb)", &page_id, &insecure_action);
+  g_signal_emit (shell, signals[SENSITIVE_FORM_FOCUSED], 0,
+                 page_id, insecure_action);
+  g_variant_unref (variant);
+}
+
 static void
 history_service_query_urls_cb (EphyHistoryService *service,
                                gboolean            success,
@@ -155,6 +198,9 @@ history_service_query_urls_cb (EphyHistoryService *service,
 
     ephy_web_extension_proxy_history_set_urls (web_extension, urls);
   }
+
+  for (l = urls; l; l = g_list_next (l))
+    ephy_embed_shell_schedule_thumbnail_update (shell, (EphyHistoryURL *)l->data);
 }
 
 static void
@@ -163,12 +209,7 @@ ephy_embed_shell_update_overview_urls (EphyEmbedShell *shell)
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   EphyHistoryQuery *query;
 
-  query = ephy_history_query_new ();
-  query->sort_type = EPHY_HISTORY_SORT_MOST_VISITED;
-  query->limit = EPHY_ABOUT_OVERVIEW_MAX_ITEMS;
-  query->ignore_hidden = TRUE;
-  query->ignore_local = TRUE;
-
+  query = ephy_history_query_new_for_overview ();
   ephy_history_service_query_urls (priv->global_history_service, query, NULL,
                                    (EphyHistoryJobCallback)history_service_query_urls_cb,
                                    shell);
@@ -395,6 +436,56 @@ ephy_embed_shell_set_thumbnail_path (EphyEmbedShell *shell,
   }
 }
 
+typedef struct {
+  char *url;
+  time_t mtime;
+} GetSnapshotPathAsyncData;
+
+static void
+got_snapshot_path_for_url_cb (EphySnapshotService      *service,
+                              GAsyncResult             *result,
+                              GetSnapshotPathAsyncData *data)
+{
+  char *snapshot;
+  GError *error = NULL;
+
+  snapshot = ephy_snapshot_service_get_snapshot_path_for_url_finish (service, result, &error);
+  if (snapshot) {
+    ephy_embed_shell_set_thumbnail_path (ephy_embed_shell_get_default (), data->url, data->mtime, snapshot);
+    g_free (snapshot);
+  } else {
+    /* Bad luck, not something to warn about. */
+    g_info ("Failed to get snapshot for URL %s: %s", data->url, error->message);
+    g_error_free (error);
+  }
+  g_free (data->url);
+  g_free (data);
+}
+
+void
+ephy_embed_shell_schedule_thumbnail_update (EphyEmbedShell *shell,
+                                            EphyHistoryURL *url)
+{
+  EphySnapshotService *service;
+  const char *snapshot;
+
+  service = ephy_snapshot_service_get_default ();
+  snapshot = ephy_snapshot_service_lookup_cached_snapshot_path (service, url->url);
+
+  if (snapshot) {
+    ephy_embed_shell_set_thumbnail_path (shell, url->url, url->thumbnail_time, snapshot);
+  } else {
+    GetSnapshotPathAsyncData *data = g_new (GetSnapshotPathAsyncData, 1);
+
+    data->url = g_strdup (url->url);
+    data->mtime = url->thumbnail_time;
+    ephy_snapshot_service_get_snapshot_path_for_url_async (service,
+                                                           url->url, url->thumbnail_time, NULL,
+                                                           (GAsyncReadyCallback)got_snapshot_path_for_url_cb,
+                                                           data);
+  }
+}
+
 /**
  * ephy_embed_shell_get_global_history_service:
  * @shell: the #EphyEmbedShell
@@ -545,6 +636,210 @@ ftp_request_cb (WebKitURISchemeRequest *request)
   g_object_unref (app_info);
 }
 
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+static void
+https_everywhere_update_cb (HTTPSEverywhereUpdater *updater,
+                            GAsyncResult           *result)
+{
+  GError *error = NULL;
+
+  https_everywhere_updater_update_finish (updater, result, &error);
+
+  if (!error)
+    return;
+
+  if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
+      !g_error_matches (error, HTTPS_EVERYWHERE_UPDATE_ERROR, HTTPS_EVERYWHERE_UPDATE_ERROR_IN_PROGRESS) &&
+      !g_error_matches (error, HTTPS_EVERYWHERE_UPDATE_ERROR, HTTPS_EVERYWHERE_UPDATE_ERROR_NO_UPDATE_AVAILABLE))
+    g_warning ("Failed to update HTTPS Everywhere rulesets: %s", error->message);
+  g_error_free (error);
+}
+#endif
+
+static const char *
+ephy_embed_shell_ensure_adblock_data_dir (EphyEmbedShell *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  if (priv->adblock_data_dir)
+    return priv->adblock_data_dir;
+
+  /* The filters list is large, so we don't want to store a separate copy per
+   * web app, but users should otherwise be able to configure different filters
+   * per profile directory. */
+  if (priv->mode == EPHY_EMBED_SHELL_MODE_APPLICATION) {
+    char *default_dot_dir = ephy_default_dot_dir ();
+
+    priv->adblock_data_dir = g_build_filename (default_dot_dir, "adblock", NULL);
+    g_free (default_dot_dir);
+  } else
+    priv->adblock_data_dir = g_build_filename (ephy_dot_dir (), "adblock", NULL);
+  g_mkdir_with_parents (priv->adblock_data_dir, 0700);
+
+  return priv->adblock_data_dir;
+}
+
+static gboolean
+adblock_filter_file_is_valid (GFile *file)
+{
+  GFileInfo *file_info;
+  gboolean result = FALSE;
+
+  /* Now check if the local file is too old. */
+  file_info = g_file_query_info (file,
+                                 G_FILE_ATTRIBUTE_TIME_MODIFIED","G_FILE_ATTRIBUTE_STANDARD_SIZE,
+                                 G_FILE_QUERY_INFO_NONE,
+                                 NULL,
+                                 NULL);
+  if (file_info) {
+    if (g_file_info_get_size (file_info) > 0) {
+      GTimeVal current_time;
+      GTimeVal mod_time;
+
+      g_get_current_time (&current_time);
+      g_file_info_get_modification_time (file_info, &mod_time);
+
+      if (current_time.tv_sec > mod_time.tv_sec) {
+        gint64 expire_time = mod_time.tv_sec + ADBLOCK_FILTER_UPDATE_FREQUENCY;
+
+        result = current_time.tv_sec < expire_time;
+      }
+    }
+    g_object_unref (file_info);
+  }
+
+  return result;
+}
+
+typedef struct {
+  EphyEmbedShell *shell;
+
+  GFile *filter_file;
+  GFile *tmp_file;
+} AdblockFilterRetrieveData;
+
+static AdblockFilterRetrieveData *
+adblock_filter_retrieve_data_new (EphyEmbedShell *shell,
+                                  GFile          *filter_file)
+{
+  AdblockFilterRetrieveData* data;
+  char *path, *tmp_path;
+
+  data = g_slice_new (AdblockFilterRetrieveData);
+  data->shell = g_object_ref (shell);
+  data->filter_file = g_object_ref (filter_file);
+
+  path = g_file_get_path (filter_file);
+  tmp_path = g_strdup_printf ("%s.tmp", path);
+  g_free (path);
+  data->tmp_file = g_file_new_for_path (tmp_path);
+  g_free (tmp_path);
+
+  return data;
+}
+
+static void
+adblock_filter_retrieve_data_free (AdblockFilterRetrieveData *data)
+{
+  g_object_unref (data->shell);
+  g_object_unref (data->filter_file);
+  g_object_unref (data->tmp_file);
+
+  g_slice_free (AdblockFilterRetrieveData, data);
+}
+
+static void
+ephy_embed_shell_retrieve_filter_file_finished (GFile                     *src,
+                                                GAsyncResult              *result,
+                                                AdblockFilterRetrieveData *data)
+{
+  GError *error = NULL;
+
+  if (!g_file_copy_finish (src, result, &error) ||
+      !g_file_move (data->tmp_file, data->filter_file, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &error)) {
+    GFileOutputStream *stream;
+
+    /* If failed to retrieve, create an empty file if it doesn't exist to unblock extensions */
+    stream = g_file_create (data->filter_file, G_FILE_CREATE_NONE, NULL, NULL);
+    g_object_unref (stream);
+
+    if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      g_warning ("Error retrieving filter: %s\n", error->message);
+    g_error_free (error);
+  }
+
+  adblock_filter_retrieve_data_free (data);
+}
+
+static void
+ephy_embed_shell_retrieve_filter_file (EphyEmbedShell *shell,
+                                       GFile          *file)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+  GFile *src = g_file_new_for_uri (ADBLOCK_FILTER_URL);
+  AdblockFilterRetrieveData *data;
+
+  if (!priv->uri_tester_update_cancellable)
+    priv->uri_tester_update_cancellable = g_cancellable_new ();
+
+  data = adblock_filter_retrieve_data_new (shell, file);
+
+  g_file_copy_async (src, data->tmp_file,
+                     G_FILE_COPY_OVERWRITE,
+                     G_PRIORITY_DEFAULT,
+                     priv->uri_tester_update_cancellable,
+                     NULL, NULL,
+                     (GAsyncReadyCallback)ephy_embed_shell_retrieve_filter_file_finished,
+                     data);
+
+  g_object_unref (src);
+}
+
+static void
+ephy_embed_shell_update_adblock_filter_file (EphyEmbedShell *shell)
+{
+  GFile *filter_file;
+
+  filter_file = ephy_uri_tester_get_adblock_filer_file (ephy_embed_shell_ensure_adblock_data_dir (shell));
+
+  if (!adblock_filter_file_is_valid (filter_file))
+    ephy_embed_shell_retrieve_filter_file (shell, filter_file);
+
+  g_object_unref (filter_file);
+}
+
+static void
+ephy_embed_shell_update_uri_tester (EphyEmbedShell *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  ephy_embed_shell_update_adblock_filter_file (shell);
+
+  if (priv->mode != EPHY_EMBED_SHELL_MODE_TEST &&
+      priv->mode != EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER) {
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+    HTTPSEverywhereContext *context;
+    HTTPSEverywhereUpdater *updater;
+#endif
+
+    if (!priv->uri_tester_update_cancellable)
+      priv->uri_tester_update_cancellable = g_cancellable_new ();
+
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+    /* We might want to be smarter about this in the future. For now,
+     * trigger an update of the rulesets once each time Epiphany is started. */
+    context = https_everywhere_context_new ();
+    updater = https_everywhere_updater_new (context);
+    https_everywhere_updater_update (updater,
+                                     priv->uri_tester_update_cancellable,
+                                     (GAsyncReadyCallback)https_everywhere_update_cb,
+                                     NULL);
+    g_object_unref (context);
+    g_object_unref (updater);
+#endif
+  }
+}
+
 static void
 web_extension_destroyed (EphyEmbedShell *shell,
                          GObject        *web_extension)
@@ -585,9 +880,10 @@ initialize_web_extensions (WebKitWebContext *web_context,
   address = priv->dbus_server ? g_dbus_server_get_client_address (priv->dbus_server) : NULL;
 
   private_profile = EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (priv->mode);
-  user_data = g_variant_new ("(mssb)",
+  user_data = g_variant_new ("(msssb)",
                              address,
                              ephy_dot_dir (),
+                             ephy_embed_shell_ensure_adblock_data_dir (shell),
                              private_profile);
   webkit_web_context_set_web_extensions_initialization_user_data (web_context, user_data);
 }
@@ -606,14 +902,11 @@ new_connection_cb (GDBusServer     *server,
                    GDBusConnection *connection,
                    EphyEmbedShell  *shell)
 {
-  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   EphyWebExtensionProxy *extension;
 
   extension = ephy_web_extension_proxy_new (connection);
   ephy_embed_shell_watch_web_extension (shell, extension);
 
-  ephy_uri_tester_handle_new_dbus_connection (priv->uri_tester, connection);
-
   g_signal_connect_object (extension, "page-created",
                            G_CALLBACK (web_extension_page_created), shell, 0);
 
@@ -738,9 +1031,9 @@ ephy_embed_shell_startup (GApplication *application)
 
   G_APPLICATION_CLASS (ephy_embed_shell_parent_class)->startup (application);
 
-  ephy_embed_shell_create_web_context (embed_shell);
+  ephy_embed_shell_update_uri_tester (embed_shell);
 
-  priv->uri_tester = ephy_uri_tester_new ();
+  ephy_embed_shell_create_web_context (embed_shell);
 
   ephy_embed_shell_setup_web_extensions_server (shell);
 
@@ -766,6 +1059,12 @@ ephy_embed_shell_startup (GApplication *application)
                     G_CALLBACK (web_extension_form_auth_data_message_received_cb),
                     shell);
 
+  webkit_user_content_manager_register_script_message_handler (priv->user_content,
+                                                               "sensitiveFormFocused");
+  g_signal_connect (priv->user_content, "script-message-received::sensitiveFormFocused",
+                    G_CALLBACK (web_extension_sensitive_form_focused_message_received_cb),
+                    shell);
+
   webkit_user_content_manager_register_script_message_handler (priv->user_content,
                                                                "aboutApps");
   g_signal_connect (priv->user_content, "script-message-received::aboutApps",
@@ -924,6 +1223,7 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
   GApplicationClass *application_class = G_APPLICATION_CLASS (klass);
 
   object_class->dispose = ephy_embed_shell_dispose;
+  object_class->finalize = ephy_embed_shell_finalize;
   object_class->set_property = ephy_embed_shell_set_property;
   object_class->get_property = ephy_embed_shell_get_property;
   object_class->constructed = ephy_embed_shell_constructed;
@@ -1031,6 +1331,23 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
                   G_TYPE_UINT64,
                   G_TYPE_STRING,
                   G_TYPE_STRING);
+
+  /**
+   * EphyEmbedShell::sensitive-form-focused
+   * @shell: the #EphyEmbedShell
+   * @page_id: the identifier of the web page
+   * @insecure_action: whether the target of the form is http://
+   *
+   * Emitted when a form in a web page gains focus.
+   */
+  signals[SENSITIVE_FORM_FOCUSED] =
+    g_signal_new ("sensitive-form-focused",
+                  EPHY_TYPE_EMBED_SHELL,
+                  G_SIGNAL_RUN_FIRST,
+                  0, NULL, NULL, NULL,
+                  G_TYPE_NONE, 2,
+                  G_TYPE_UINT64,
+                  G_TYPE_BOOLEAN);
 }
 
 /**
