@@ -1,45 +1,50 @@
+/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /*
  *  Copyright © 2011 Igalia S.L.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This file is part of Epiphany.
+ *
+ *  Epiphany is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  *
  *  Some parts of this file based on the Midori's 'adblock' extension,
  *  licensed with the GNU Lesser General Public License 2.1, Copyright
  *  (C) 2009-2010 Christian Dywan <christian@twotoasts.de> and 2009
  *  Alexander Butenko <a.butenka@gmail.com>. Check Midori's web site
  *  at http://www.twotoasts.de
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 #include "ephy-uri-tester.h"
 
 #include "ephy-debug.h"
+#include "ephy-uri-tester-shared.h"
 
 #include <gio/gio.h>
 #include <glib/gstdio.h>
+#include <libsoup/soup.h>
 #include <string.h>
 
-#define DEFAULT_FILTER_URL "https://easylist-downloads.adblockplus.org/easylist.txt"
-#define FILTERS_LIST_FILENAME "filters.list"
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+#include <httpseverywhere.h>
+#endif
+
 #define SIGNATURE_SIZE 8
-#define UPDATE_FREQUENCY 24 * 60 * 60 /* In seconds */
 
 struct _EphyUriTester {
   GObject parent_instance;
 
-  GSList *filters;
-  char *data_dir;
+  char *adblock_data_dir;
 
   GHashTable *pattern;
   GHashTable *keys;
@@ -58,12 +63,19 @@ struct _EphyUriTester {
   GRegex *regex_pattern;
   GRegex *regex_subdocument;
   GRegex *regex_frame_add;
+
+  GMainLoop *load_loop;
+  gboolean adblock_loaded;
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+  gboolean https_everywhere_loaded;
+
+  HTTPSEverywhereContext *https_everywhere_context;
+#endif
 };
 
 enum {
   PROP_0,
-  PROP_FILTERS,
-  PROP_BASE_DATA_DIR,
+  PROP_ADBLOCK_DATA_DIR,
   LAST_PROP
 };
 
@@ -71,182 +83,9 @@ static GParamSpec *obj_properties[LAST_PROP];
 
 G_DEFINE_TYPE (EphyUriTester, ephy_uri_tester, G_TYPE_OBJECT)
 
-/* Private functions. */
-
 static GString *
 ephy_uri_tester_fixup_regexp (const char *prefix, char *src);
 
-static void
-ephy_uri_tester_parse_file_at_uri (EphyUriTester *tester, const char *fileuri);
-
-static char *
-ephy_uri_tester_ensure_data_dir (const char *base_data_dir)
-{
-  char *folder;
-
-  /* Ensure adblock's dir is there. */
-  folder = g_build_filename (base_data_dir, "adblock", NULL);
-  g_mkdir_with_parents (folder, 0700);
-
-  return folder;
-}
-
-static char *
-ephy_uri_tester_get_fileuri_for_url (EphyUriTester *tester,
-                                     const char    *url)
-{
-  char *filename = NULL;
-  char *path = NULL;
-  char *uri = NULL;
-
-  if (!strncmp (url, "file", 4))
-    return g_strndup (url + 7, strlen (url) - 7);
-
-  filename = g_compute_checksum_for_string (G_CHECKSUM_MD5, url, -1);
-
-  path = g_build_filename (tester->data_dir, filename, NULL);
-  uri = g_filename_to_uri (path, NULL, NULL);
-
-  g_free (filename);
-  g_free (path);
-
-  return uri;
-}
-
-static void
-ephy_uri_tester_retrieve_filter (EphyUriTester *tester,
-                                 const char    *url,
-                                 const char    *fileuri)
-{
-  GFile *src;
-  GFile *dest;
-  GError *error = NULL;
-
-  g_return_if_fail (EPHY_IS_URI_TESTER (tester));
-  g_return_if_fail (url != NULL);
-  g_return_if_fail (fileuri != NULL);
-
-  src = g_file_new_for_uri (url);
-  dest = g_file_new_for_uri (fileuri);
-
-  g_file_copy (src, dest,
-               G_FILE_COPY_OVERWRITE,
-               NULL, NULL, NULL,
-               &error);
-
-  if (error != NULL) {
-    g_warning ("Error retrieving adblock filter: %s\n", error->message);
-    g_error_free (error);
-  } else {
-    ephy_uri_tester_parse_file_at_uri (tester, fileuri);
-  }
-
-  g_object_unref (src);
-  g_object_unref (dest);
-}
-
-static gboolean
-ephy_uri_tester_filter_is_valid (const char *fileuri)
-{
-  GFile *file = NULL;
-  GFileInfo *file_info = NULL;
-  gboolean result;
-
-  /* Now check if the local file is too old. */
-  file = g_file_new_for_uri (fileuri);
-  file_info = g_file_query_info (file,
-                                 G_FILE_ATTRIBUTE_TIME_MODIFIED,
-                                 G_FILE_QUERY_INFO_NONE,
-                                 NULL,
-                                 NULL);
-  result = FALSE;
-  if (file_info) {
-    GTimeVal current_time;
-    GTimeVal mod_time;
-
-    g_get_current_time (&current_time);
-    g_file_info_get_modification_time (file_info, &mod_time);
-
-    if (current_time.tv_sec > mod_time.tv_sec) {
-      gint64 expire_time = mod_time.tv_sec + UPDATE_FREQUENCY;
-      result = current_time.tv_sec < expire_time;
-    }
-    g_object_unref (file_info);
-  }
-
-  g_object_unref (file);
-
-  return result;
-}
-
-static void
-ephy_uri_tester_load_patterns (EphyUriTester *tester)
-{
-  GSList *filter = NULL;
-  char *url = NULL;
-  char *fileuri = NULL;
-
-  /* Load patterns from the list of filters. */
-  for (filter = tester->filters; filter; filter = g_slist_next (filter)) {
-    url = (char *)filter->data;
-    fileuri = ephy_uri_tester_get_fileuri_for_url (tester, url);
-
-    if (!ephy_uri_tester_filter_is_valid (fileuri))
-      ephy_uri_tester_retrieve_filter (tester, url, fileuri);
-    else
-      ephy_uri_tester_parse_file_at_uri (tester, fileuri);
-
-    g_free (fileuri);
-  }
-}
-
-static void
-ephy_uri_tester_load_filters (EphyUriTester *tester)
-{
-  GSList *list = NULL;
-  char *filepath = NULL;
-
-  filepath = g_build_filename (tester->data_dir, FILTERS_LIST_FILENAME, NULL);
-
-  if (g_file_test (filepath, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
-    GFile *file = NULL;
-    char *contents = NULL;
-    gsize length = 0;
-    GError *error = NULL;
-
-    file = g_file_new_for_path (filepath);
-    if (g_file_load_contents (file, NULL, &contents, &length, NULL, &error)) {
-      char **urls_array = NULL;
-      char *url = NULL;
-      int i = 0;
-
-      urls_array = g_strsplit (contents, ";", -1);
-      for (i = 0; urls_array [i]; i++) {
-        url = g_strstrip (g_strdup (urls_array[i]));
-        if (!g_str_equal (url, ""))
-          list = g_slist_prepend (list, url);
-      }
-      g_strfreev (urls_array);
-
-      g_free (contents);
-    }
-
-    if (error) {
-      g_warning ("Error loading filters from %s: %s", filepath, error->message);
-      g_error_free (error);
-    }
-
-    g_object_unref (file);
-  } else {
-    /* No file exists yet, so use the default filter and save it. */
-    list = g_slist_prepend (list, g_strdup (DEFAULT_FILTER_URL));
-  }
-
-  g_free (filepath);
-
-  ephy_uri_tester_set_filters (tester, g_slist_reverse (list));
-}
-
 static inline int
 ephy_uri_tester_check_rule (EphyUriTester *tester,
                             GRegex        *regex,
@@ -668,43 +507,193 @@ ephy_uri_tester_parse_line (EphyUriTester *tester,
 }
 
 static void
-ephy_uri_tester_parse_file_at_uri (EphyUriTester *tester, const char *fileuri)
+ephy_uri_tester_adblock_loaded (EphyUriTester *tester)
+{
+  tester->adblock_loaded = TRUE;
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+  if (tester->https_everywhere_loaded)
+    g_main_loop_quit (tester->load_loop);
+#else
+  g_main_loop_quit (tester->load_loop);
+#endif
+}
+
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+static void
+ephy_uri_tester_https_everywhere_loaded (EphyUriTester *tester)
+{
+  tester->https_everywhere_loaded = TRUE;
+  if (tester->adblock_loaded)
+    g_main_loop_quit (tester->load_loop);
+}
+#endif
+
+static void
+file_parse_cb (GDataInputStream *stream, GAsyncResult *result, EphyUriTester *tester)
 {
-  GFile *file;
-  GFileInputStream *stream;
-  GDataInputStream *data_stream = NULL;
   char *line;
   GError *error = NULL;
 
-  file = g_file_new_for_uri (fileuri);
-  stream = g_file_read (file, NULL, &error);
-  g_object_unref (file);
+  line = g_data_input_stream_read_line_finish (stream, result, NULL, &error);
+  if (!line) {
+    if (error) {
+      LOG ("Error parsing file: %s\n", error->message);
+      g_error_free (error);
+    }
 
+    ephy_uri_tester_adblock_loaded (tester);
+    return;
+  }
+
+  ephy_uri_tester_parse_line (tester, line, FALSE);
+  g_free (line);
+
+  g_data_input_stream_read_line_async (stream, G_PRIORITY_DEFAULT_IDLE, NULL,
+                                       (GAsyncReadyCallback)file_parse_cb, tester);
+}
+
+static void
+file_read_cb (GFile *file, GAsyncResult *result, EphyUriTester *tester)
+{
+  GFileInputStream *stream;
+  GDataInputStream *data_stream;
+  GError *error = NULL;
+
+  stream = g_file_read_finish (file, result, &error);
   if (!stream) {
-    g_warning ("Error opening %s for parsing: %s\n", fileuri, error->message);
+    char *path;
+
+    path = g_file_get_path (file);
+    LOG ("Error opening file %s for parsing: %s\n", path, error->message);
+    g_free (path);
     g_error_free (error);
+
+    ephy_uri_tester_adblock_loaded (tester);
     return;
   }
 
   data_stream = g_data_input_stream_new (G_INPUT_STREAM (stream));
   g_object_unref (stream);
 
-  for (;;) {
-    line = g_data_input_stream_read_line (data_stream, NULL, NULL, &error);
-    if (!line) {
-      if (error) {
-        g_warning ("Error parsing file: %s\n", error->message);
-        g_error_free (error);
-      }
+  g_data_input_stream_read_line_async (data_stream, G_PRIORITY_DEFAULT_IDLE, NULL,
+                                       (GAsyncReadyCallback)file_parse_cb, tester);
+  g_object_unref (data_stream);
+}
 
-      break;
-    }
+static gboolean
+ephy_uri_tester_block_uri (EphyUriTester *tester,
+                          const char    *req_uri,
+                          const char    *page_uri)
+{
+  /* check whitelisting rules before the normal ones */
+  if (ephy_uri_tester_is_matched (tester, NULL, req_uri, page_uri, TRUE))
+    return FALSE;
+  return ephy_uri_tester_is_matched (tester, NULL, req_uri, page_uri, FALSE);
+}
 
-    ephy_uri_tester_parse_line (tester, line, FALSE);
-    g_free (line);
+char *
+ephy_uri_tester_rewrite_uri (EphyUriTester    *tester,
+                             const char       *request_uri,
+                             const char       *page_uri,
+                             EphyUriTestFlags  flags)
+{
+  /* Should we block the URL outright? */
+  if ((flags & EPHY_URI_TEST_ADBLOCK) &&
+      ephy_uri_tester_block_uri (tester, request_uri, page_uri)) {
+    g_debug ("Request '%s' blocked (page: '%s')", request_uri, page_uri);
+
+    return NULL;
   }
 
-  g_object_unref (data_stream);
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+  if ((flags & EPHY_URI_TEST_HTTPS_EVERYWHERE) && https_everywhere_context_get_initialized (tester->https_everywhere_context))
+    return https_everywhere_context_rewrite (tester->https_everywhere_context, request_uri);
+#endif
+
+  return g_strdup (request_uri);
+}
+
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+static void
+https_everywhere_context_init_cb (HTTPSEverywhereContext *context,
+                                  GAsyncResult           *res,
+                                  EphyUriTester          *tester)
+{
+  GError *error = NULL;
+
+  https_everywhere_context_init_finish (context, res, &error);
+
+  if (error) {
+    g_warning ("Failed to initialize HTTPS Everywhere context: %s", error->message);
+    g_error_free (error);
+  }
+
+  ephy_uri_tester_https_everywhere_loaded (tester);
+}
+#endif
+
+static void
+adblock_file_monitor_changed (GFileMonitor     *monitor,
+                              GFile            *file,
+                              GFile            *other_file,
+                              GFileMonitorEvent event_type,
+                              EphyUriTester    *tester)
+{
+  if (event_type != G_FILE_MONITOR_EVENT_RENAMED)
+    return;
+
+  g_signal_handlers_disconnect_by_func (monitor, adblock_file_monitor_changed, tester);
+  g_file_read_async (other_file, G_PRIORITY_DEFAULT_IDLE, NULL,
+                     (GAsyncReadyCallback)file_read_cb,
+                     tester);
+}
+
+static void
+ephy_uri_tester_load_sync (GTask         *task,
+                           EphyUriTester *tester)
+{
+  GMainContext *context;
+  GFile *filter_file;
+  GFileMonitor *monitor = NULL;
+
+  context = g_main_context_new ();
+  tester->load_loop = g_main_loop_new (context, FALSE);
+  g_main_context_push_thread_default (context);
+
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+  tester->https_everywhere_context = https_everywhere_context_new ();
+  https_everywhere_context_init (tester->https_everywhere_context, NULL,
+                                 (GAsyncReadyCallback)https_everywhere_context_init_cb,
+                                 tester);
+#endif
+
+  filter_file = ephy_uri_tester_get_adblock_filer_file (tester->adblock_data_dir);
+  if (!g_file_query_exists (filter_file, NULL)) {
+    GError *error = NULL;
+
+    monitor = g_file_monitor_file (filter_file, G_FILE_MONITOR_WATCH_MOVES, NULL, &error);
+    if (monitor) {
+       g_signal_connect (monitor, "changed", G_CALLBACK (adblock_file_monitor_changed), tester);
+    } else {
+      g_warning ("Failed to monitor adblock file: %s\n", error->message);
+      g_error_free (error);
+      ephy_uri_tester_adblock_loaded (tester);
+    }
+  } else {
+    g_file_read_async (filter_file, G_PRIORITY_DEFAULT_IDLE, NULL,
+                       (GAsyncReadyCallback)file_read_cb,
+                       tester);
+  }
+  g_object_unref (filter_file);
+
+  g_main_loop_run (tester->load_loop);
+
+  if (monitor)
+    g_object_unref (monitor);
+  g_main_context_pop_thread_default (context);
+  g_main_context_unref (context);
+
+  g_task_return_boolean (task, TRUE);
 }
 
 static void
@@ -712,7 +701,6 @@ ephy_uri_tester_init (EphyUriTester *tester)
 {
   LOG ("EphyUriTester initializing %p", tester);
 
-  tester->filters = NULL;
   tester->pattern = g_hash_table_new_full (g_str_hash, g_str_equal,
                                            (GDestroyNotify)g_free,
                                            (GDestroyNotify)g_regex_unref);
@@ -760,17 +748,6 @@ ephy_uri_tester_init (EphyUriTester *tester)
                                          NULL);
 }
 
-static void
-ephy_uri_tester_constructed (GObject *object)
-{
-  EphyUriTester *tester = EPHY_URI_TESTER (object);
-
-  G_OBJECT_CLASS (ephy_uri_tester_parent_class)->constructed (object);
-
-  ephy_uri_tester_load_filters (tester);
-  ephy_uri_tester_load_patterns (tester);
-}
-
 static void
 ephy_uri_tester_set_property (GObject      *object,
                               guint         prop_id,
@@ -780,11 +757,8 @@ ephy_uri_tester_set_property (GObject      *object,
   EphyUriTester *tester = EPHY_URI_TESTER (object);
 
   switch (prop_id) {
-    case PROP_FILTERS:
-      ephy_uri_tester_set_filters (tester, (GSList *)g_value_get_pointer (value));
-      break;
-    case PROP_BASE_DATA_DIR:
-      tester->data_dir = ephy_uri_tester_ensure_data_dir (g_value_get_string (value));
+    case PROP_ADBLOCK_DATA_DIR:
+      tester->adblock_data_dir = g_value_dup_string (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -799,8 +773,7 @@ ephy_uri_tester_finalize (GObject *object)
 
   LOG ("EphyUriTester finalizing %p", object);
 
-  g_slist_free_full (tester->filters, g_free);
-  g_free (tester->data_dir);
+  g_free (tester->adblock_data_dir);
 
   g_hash_table_destroy (tester->pattern);
   g_hash_table_destroy (tester->keys);
@@ -820,6 +793,10 @@ ephy_uri_tester_finalize (GObject *object)
   g_regex_unref (tester->regex_subdocument);
   g_regex_unref (tester->regex_frame_add);
 
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+  g_clear_object (&tester->https_everywhere_context);
+#endif
+
   G_OBJECT_CLASS (ephy_uri_tester_parent_class)->finalize (object);
 }
 
@@ -829,19 +806,12 @@ ephy_uri_tester_class_init (EphyUriTesterClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->set_property = ephy_uri_tester_set_property;
-  object_class->constructed = ephy_uri_tester_constructed;
   object_class->finalize = ephy_uri_tester_finalize;
 
-  obj_properties[PROP_FILTERS] =
-    g_param_spec_pointer ("filters",
-                          "filters",
-                          "filters",
-                          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
-
-  obj_properties[PROP_BASE_DATA_DIR] =
-    g_param_spec_string ("base-data-dir",
-                         "Base data dir",
-                         "The base dir where to create the adblock data dir",
+  obj_properties[PROP_ADBLOCK_DATA_DIR] =
+    g_param_spec_string ("adblock-data-dir",
+                         "Adblock data dir",
+                         "The adblock data dir",
                          NULL,
                          G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
@@ -849,29 +819,26 @@ ephy_uri_tester_class_init (EphyUriTesterClass *klass)
 }
 
 EphyUriTester *
-ephy_uri_tester_new (const char *base_data_dir)
+ephy_uri_tester_new (const char *adblock_data_dir)
 {
-  g_return_val_if_fail (base_data_dir != NULL, NULL);
-
-  return g_object_new (EPHY_TYPE_URI_TESTER, "base-data-dir", base_data_dir, NULL);
-}
-
-gboolean
-ephy_uri_tester_test_uri (EphyUriTester *tester,
-                          const char    *req_uri,
-                          const char    *page_uri)
-{
-  /* check whitelisting rules before the normal ones */
-  if (ephy_uri_tester_is_matched (tester, NULL, req_uri, page_uri, TRUE))
-    return FALSE;
-  return ephy_uri_tester_is_matched (tester, NULL, req_uri, page_uri, FALSE);
+  return EPHY_URI_TESTER (g_object_new (EPHY_TYPE_URI_TESTER, "adblock-data-dir", adblock_data_dir, NULL));
 }
 
 void
-ephy_uri_tester_set_filters (EphyUriTester *tester, GSList *filters)
+ephy_uri_tester_load (EphyUriTester *tester)
 {
-  if (tester->filters)
-    g_slist_free_full (tester->filters, g_free);
+  GTask *task;
 
-  tester->filters = filters;
+  g_return_if_fail (EPHY_IS_URI_TESTER (tester));
+
+  if (tester->adblock_loaded
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+      && tester->https_everywhere_loaded
+#endif
+     )
+    return;
+
+  task = g_task_new (tester, NULL, NULL, NULL);
+  g_task_run_in_thread_sync (task, (GTaskThreadFunc)ephy_uri_tester_load_sync);
+  g_object_unref (task);
 }
