@@ -30,7 +30,8 @@
 #include "ephy-hosts-manager.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
-#include "ephy-uri-tester-proxy.h"
+#include "ephy-uri-helpers.h"
+#include "ephy-uri-tester.h"
 #include "ephy-web-dom-utils.h"
 #include "ephy-web-overview.h"
 
@@ -39,8 +40,6 @@
 #include <libsoup/soup.h>
 #include <string.h>
 #include <webkit2/webkit-web-extension.h>
-#define WEBKIT_DOM_USE_UNSTABLE_API
-#include <webkitdom/WebKitDOMDOMWindowUnstable.h>
 #include <JavaScriptCore/JavaScript.h>
 
 struct _EphyWebExtension {
@@ -50,12 +49,13 @@ struct _EphyWebExtension {
   gboolean initialized;
 
   GDBusConnection *dbus_connection;
+  GArray *page_created_signals_pending;
 
   EphyFormAuthDataCache *form_auth_data_cache;
   GHashTable *form_auth_data_save_requests;
   EphyWebOverviewModel *overview_model;
   EphyHostsManager *hosts_manager;
-  EphyUriTesterProxy *uri_tester;
+  EphyUriTester *uri_tester;
 };
 
 static const char introspection_xml[] =
@@ -105,6 +105,76 @@ static const char introspection_xml[] =
 
 G_DEFINE_TYPE (EphyWebExtension, ephy_web_extension, G_TYPE_OBJECT)
 
+static gboolean
+should_use_https_everywhere (const char *request_uri,
+                             const char *redirected_uri)
+{
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+  SoupURI *request_soup_uri;
+  SoupURI *redirected_soup_uri;
+  gboolean result = TRUE;
+
+  request_soup_uri = soup_uri_new (request_uri);
+  if (request_soup_uri->scheme != SOUP_URI_SCHEME_HTTP) {
+    soup_uri_free (request_soup_uri);
+    return FALSE;
+  }
+
+  if (!redirected_uri) {
+    soup_uri_free (request_soup_uri);
+    return TRUE;
+  }
+
+  redirected_soup_uri = soup_uri_new (redirected_uri);
+
+  if (request_soup_uri->scheme == SOUP_URI_SCHEME_HTTP &&
+      redirected_soup_uri->scheme == SOUP_URI_SCHEME_HTTPS) {
+    /* If the server redirected us from an https:// URI to an http:// URI, we'd
+     * better not try to use HTTPS Everywhere as it'll just be a redirect loop.
+     * So now we compare ignoring scheme and port.... */
+    redirected_soup_uri->scheme = SOUP_URI_SCHEME_HTTP;
+    redirected_soup_uri->port = request_soup_uri->port;
+    result = !soup_uri_equal (request_soup_uri, redirected_soup_uri);
+  }
+
+  soup_uri_free (request_soup_uri);
+  soup_uri_free (redirected_soup_uri);
+
+  return result;
+#else
+  return FALSE;
+#endif
+}
+
+static gboolean
+should_use_adblocker (const char *request_uri,
+                      const char *page_uri)
+{
+  /* Always load the main resource. */
+  if (g_strcmp0 (request_uri, page_uri) == 0)
+    return FALSE;
+
+  /* Always load data requests, as uri_tester won't do any good here. */
+  if (g_str_has_prefix (request_uri, SOUP_URI_SCHEME_DATA))
+    return FALSE;
+
+  /* Always load about pages */
+  if (g_str_has_prefix (request_uri, "about") ||
+      g_str_has_prefix (request_uri, "ephy-about"))
+    return FALSE;
+
+  /* Always load resources */
+  if (g_str_has_prefix (request_uri, "resource://") ||
+      g_str_has_prefix (request_uri, "ephy-resource://"))
+    return FALSE;
+
+  /* Always load local files */
+  if (g_str_has_prefix (request_uri, "file://"))
+    return FALSE;
+
+  return TRUE;
+}
+
 static gboolean
 web_page_send_request (WebKitWebPage     *web_page,
                        WebKitURIRequest  *request,
@@ -112,8 +182,17 @@ web_page_send_request (WebKitWebPage     *web_page,
                        EphyWebExtension  *extension)
 {
   const char *request_uri;
+  const char *redirected_response_uri;
   const char *page_uri;
-  char *modified_uri;
+  char *modified_uri = NULL;
+  EphyUriTestFlags flags = EPHY_URI_TEST_ALL;
+
+  request_uri = webkit_uri_request_get_uri (request);
+  page_uri = webkit_web_page_get_uri (web_page);
+
+  if (!should_use_adblocker (request_uri, page_uri) ||
+      !g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ENABLE_ADBLOCK))
+    flags &= ~EPHY_URI_TEST_ADBLOCK;
 
   if (g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DO_NOT_TRACK)) {
     SoupMessageHeaders *headers = webkit_uri_request_get_http_headers (request);
@@ -122,19 +201,30 @@ web_page_send_request (WebKitWebPage     *web_page,
        * http://tools.ietf.org/id/draft-mayer-do-not-track-00.txt */
       soup_message_headers_append (headers, "DNT", "1");
     }
+    modified_uri = ephy_remove_tracking_from_uri (request_uri);
   }
 
-  request_uri = webkit_uri_request_get_uri (request);
-  page_uri = webkit_web_page_get_uri (web_page);
+  redirected_response_uri = redirected_response ? webkit_uri_response_get_uri (redirected_response) : NULL;
+  if (!should_use_https_everywhere (request_uri, redirected_response_uri))
+    flags &= ~EPHY_URI_TEST_HTTPS_EVERYWHERE;
 
-  modified_uri = ephy_uri_tester_proxy_maybe_rewrite_uri (extension->uri_tester,
-                                                          request_uri,
-                                                          page_uri);
+  if ((flags & EPHY_URI_TEST_ADBLOCK) || (flags & EPHY_URI_TEST_HTTPS_EVERYWHERE)) {
+    char *result;
 
-  if (strlen (modified_uri) == 0) {
-    LOG ("Refused to load %s", request_uri);
+    ephy_uri_tester_load (extension->uri_tester);
+    result = ephy_uri_tester_rewrite_uri (extension->uri_tester,
+                                          modified_uri ? modified_uri : request_uri,
+                                          page_uri, flags);
     g_free (modified_uri);
-    return TRUE;
+
+    if (!result) {
+      LOG ("Refused to load %s", request_uri);
+      return TRUE;
+    }
+
+    modified_uri = result;
+  } else if (!modified_uri) {
+    return FALSE;
   }
 
   if (g_strcmp0 (request_uri, modified_uri) != 0) {
@@ -931,6 +1021,46 @@ username_node_input_cb (WebKitDOMNode  *username_node,
   return TRUE;
 }
 
+static gboolean
+sensitive_form_focused_cb (WebKitDOMHTMLFormElement *form,
+                           WebKitDOMEvent           *dom_event,
+                           WebKitWebPage            *web_page)
+{
+  WebKitDOMDOMWindow *dom_window;
+  GVariant *variant;
+  char *message;
+  char *action;
+  gboolean insecure_action;
+
+  dom_window = webkit_dom_document_get_default_view (webkit_web_page_get_dom_document (web_page));
+  if (dom_window == NULL)
+    return FALSE;
+
+  action = webkit_dom_html_form_element_get_action (form);
+  /* The goal here is to detect insecure forms on secure pages. The action need
+   * not necessarily contain any protocol, so just assume the form is secure
+   * unless it's clearly not. Insecure forms on insecure pages will be detected
+   * in the UI process. Note that basically no websites should actually be dumb
+   * enough to trip this, but no doubt they exist somewhere.... */
+  insecure_action = action == NULL && g_str_has_prefix (action, "http://");
+
+  variant = g_variant_new ("(tb)",
+                           webkit_web_page_get_id (web_page),
+                           insecure_action);
+  message = g_variant_print (variant, FALSE);
+
+  if (!webkit_dom_dom_window_webkit_message_handlers_post_message (dom_window, "sensitiveFormFocused", message))
+    g_warning ("Error sending sensitiveFormFocused message");
+
+  g_free (action);
+  g_free (message);
+  g_object_unref (dom_window);
+  g_variant_unref (variant);
+
+  /* Hoping FALSE means "propagate" because there is absolutely no documentation for this. */
+  return FALSE;
+}
+
 static void
 form_destroyed_cb (gpointer form_auth, GObject *form)
 {
@@ -938,12 +1068,11 @@ form_destroyed_cb (gpointer form_auth, GObject *form)
 }
 
 static void
-web_page_document_loaded (WebKitWebPage    *web_page,
-                          EphyWebExtension *extension)
+web_page_form_controls_associated (WebKitWebPage    *web_page,
+                                   GPtrArray        *elements,
+                                   EphyWebExtension *extension)
 {
-  WebKitDOMHTMLCollection *forms = NULL;
   WebKitDOMDocument *document = NULL;
-  gulong forms_n;
   guint i;
 
   if (!extension->form_auth_data_cache ||
@@ -951,21 +1080,18 @@ web_page_document_loaded (WebKitWebPage    *web_page,
     return;
 
   document = webkit_web_page_get_dom_document (web_page);
-  forms = webkit_dom_document_get_forms (document);
-  forms_n = webkit_dom_html_collection_get_length (forms);
 
-  if (forms_n == 0) {
-    LOG ("No forms found.");
-    g_object_unref (forms);
-    return;
-  }
-
-  for (i = 0; i < forms_n; i++) {
+  for (i = 0; i < elements->len; ++i) {
+    WebKitDOMElement *element;
     WebKitDOMHTMLFormElement *form;
     WebKitDOMNode *username_node = NULL;
     WebKitDOMNode *password_node = NULL;
 
-    form = WEBKIT_DOM_HTML_FORM_ELEMENT (webkit_dom_html_collection_item (forms, i));
+    element = WEBKIT_DOM_ELEMENT (g_ptr_array_index (elements, i));
+    if (!WEBKIT_DOM_IS_HTML_FORM_ELEMENT (element))
+      continue;
+
+    form = WEBKIT_DOM_HTML_FORM_ELEMENT (element);
 
     /* We have a field that may be the user, and one for a password. */
     if (ephy_web_dom_utils_find_form_auth_elements (form, &username_node, &password_node)) {
@@ -1023,9 +1149,13 @@ web_page_document_loaded (WebKitWebPage    *web_page,
       g_object_weak_ref (G_OBJECT (form), form_destroyed_cb, form_auth);
     } else
       LOG ("No pre-fillable/hookable form found");
-  }
 
-  g_object_unref (forms);
+    if (ephy_web_dom_utils_form_contains_sensitive_element (form)) {
+      webkit_dom_event_target_add_event_listener (WEBKIT_DOM_EVENT_TARGET (form), "focus",
+                                                  G_CALLBACK (sensitive_form_focused_cb), TRUE,
+                                                  web_page);
+    }
+  }
 }
 
 static void
@@ -1095,6 +1225,34 @@ ephy_web_extension_emit_page_created (EphyWebExtension *extension,
   }
 }
 
+static void
+ephy_web_extension_emit_page_created_signals_pending (EphyWebExtension *extension)
+{
+  guint i;
+
+  if (!extension->page_created_signals_pending)
+    return;
+
+  for (i = 0; i < extension->page_created_signals_pending->len; i++) {
+    guint64 page_id;
+
+    page_id = g_array_index (extension->page_created_signals_pending, guint64, i);
+    ephy_web_extension_emit_page_created (extension, page_id);
+  }
+
+  g_array_free (extension->page_created_signals_pending, TRUE);
+  extension->page_created_signals_pending = NULL;
+}
+
+static void
+ephy_web_extension_queue_page_created_signal_emission (EphyWebExtension *extension,
+                                                       guint64           page_id)
+{
+  if (!extension->page_created_signals_pending)
+    extension->page_created_signals_pending = g_array_new (FALSE, FALSE, sizeof (guint64));
+  extension->page_created_signals_pending = g_array_append_val (extension->page_created_signals_pending, page_id);
+}
+
 static void
 ephy_web_extension_page_created_cb (EphyWebExtension *extension,
                                     WebKitWebPage    *web_page)
@@ -1102,20 +1260,23 @@ ephy_web_extension_page_created_cb (EphyWebExtension *extension,
   guint64 page_id;
 
   page_id = webkit_web_page_get_id (web_page);
-  ephy_web_extension_emit_page_created (extension, page_id);
+  if (extension->dbus_connection)
+    ephy_web_extension_emit_page_created (extension, page_id);
+  else
+    ephy_web_extension_queue_page_created_signal_emission (extension, page_id);
 
   g_signal_connect (web_page, "send-request",
                     G_CALLBACK (web_page_send_request),
                     extension);
-  g_signal_connect (web_page, "document-loaded",
-                    G_CALLBACK (web_page_document_loaded),
-                    extension);
   g_signal_connect (web_page, "notify::uri",
                     G_CALLBACK (web_page_uri_changed),
                     extension);
   g_signal_connect (web_page, "context-menu",
                     G_CALLBACK (web_page_context_menu),
                     extension);
+  g_signal_connect (web_page, "form-controls-associated",
+                    G_CALLBACK (web_page_form_controls_associated),
+                    extension);
 }
 
 static WebKitWebPage *
@@ -1287,6 +1448,7 @@ ephy_web_extension_dispose (GObject *object)
 {
   EphyWebExtension *extension = EPHY_WEB_EXTENSION (object);
 
+  g_clear_object (&extension->uri_tester);
   g_clear_object (&extension->overview_model);
   g_clear_object (&extension->hosts_manager);
 
@@ -1298,6 +1460,11 @@ ephy_web_extension_dispose (GObject *object)
     extension->form_auth_data_save_requests = NULL;
   }
 
+  if (extension->page_created_signals_pending) {
+    g_array_free (extension->page_created_signals_pending, TRUE);
+    extension->page_created_signals_pending = NULL;
+  }
+
   g_clear_object (&extension->dbus_connection);
   g_clear_object (&extension->extension);
 
@@ -1331,6 +1498,45 @@ ephy_web_extension_get (void)
   return EPHY_WEB_EXTENSION (g_once (&once_init, ephy_web_extension_create_instance, NULL));
 }
 
+static void
+dbus_connection_created_cb (GObject          *source_object,
+                            GAsyncResult     *result,
+                            EphyWebExtension *extension)
+{
+  static GDBusNodeInfo *introspection_data = NULL;
+  GDBusConnection *connection;
+  guint registration_id;
+  GError *error = NULL;
+
+  if (!introspection_data)
+    introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
+
+  connection = g_dbus_connection_new_for_address_finish (result, &error);
+  if (error) {
+    g_warning ("Failed to connect to UI process: %s", error->message);
+    g_error_free (error);
+    return;
+  }
+
+  registration_id =
+    g_dbus_connection_register_object (connection,
+                                       EPHY_WEB_EXTENSION_OBJECT_PATH,
+                                       introspection_data->interfaces[0],
+                                       &interface_vtable,
+                                       extension,
+                                       NULL,
+                                       &error);
+  if (!registration_id) {
+    g_warning ("Failed to register web extension object: %s\n", error->message);
+    g_error_free (error);
+    g_object_unref (connection);
+    return;
+  }
+
+  extension->dbus_connection = connection;
+  ephy_web_extension_emit_page_created_signals_pending (extension);
+}
+
 static gboolean
 authorize_authenticated_peer_cb (GDBusAuthObserver *observer,
                                  GIOStream         *stream,
@@ -1344,12 +1550,10 @@ void
 ephy_web_extension_initialize (EphyWebExtension   *extension,
                                WebKitWebExtension *wk_extension,
                                const char         *server_address,
+                               const char         *adblock_data_dir,
                                gboolean            is_private_profile)
 {
-  static GDBusNodeInfo *introspection_data = NULL;
   GDBusAuthObserver *observer;
-  guint registration_id;
-  GError *error = NULL;
 
   g_return_if_fail (EPHY_IS_WEB_EXTENSION (extension));
 
@@ -1372,35 +1576,13 @@ ephy_web_extension_initialize (EphyWebExtension   *extension,
   g_signal_connect (observer, "authorize-authenticated-peer",
                     G_CALLBACK (authorize_authenticated_peer_cb), extension);
 
-  /* Sync because this has to be ready before the first URI request, since
-   * WebKit does not provide any async way to rewrite URI requests. */
-  extension->dbus_connection =
-      g_dbus_connection_new_for_address_sync (server_address,
-                                              G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT,
-                                              observer,
-                                              NULL,
-                                              &error);
+  g_dbus_connection_new_for_address (server_address,
+                                     G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT,
+                                     observer,
+                                     NULL,
+                                     (GAsyncReadyCallback)dbus_connection_created_cb,
+                                     extension);
   g_object_unref (observer);
 
-  /* Fatal. */
-  if (error)
-    g_error ("Failed to connect to UI process: %s", error->message);
-
-  if (!introspection_data)
-    introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
-
-  registration_id =
-    g_dbus_connection_register_object (extension->dbus_connection,
-                                       EPHY_WEB_EXTENSION_OBJECT_PATH,
-                                       introspection_data->interfaces[0],
-                                       &interface_vtable,
-                                       extension,
-                                       NULL,
-                                       &error);
-
-  /* Fatal. */
-  if (!registration_id)
-    g_error ("Failed to register web extension object: %s\n", error->message);
-
-  extension->uri_tester = ephy_uri_tester_proxy_new (extension->dbus_connection);
+  extension->uri_tester = ephy_uri_tester_new (adblock_data_dir);
 }
