@@ -976,6 +976,19 @@ ephy_sanitize_filename (char *filename)
   return g_strdelimit (filename, G_DIR_SEPARATOR_S, '_');
 }
 
+void
+ephy_open_default_instance_window (void)
+{
+  GError *error = NULL;
+
+  g_spawn_command_line_async ("epiphany", &error);
+
+  if (error) {
+    g_warning ("Couldn't open default instance: %s", error->message);
+    g_error_free (error);
+  }
+}
+
 void
 ephy_open_incognito_window (const char *uri)
 {
