@@ -22,10 +22,10 @@
 
 #include "ephy-bookmarks-manager.h"
 
+#include "ephy-bookmarks-export.h"
+#include "ephy-bookmarks-import.h"
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
-#include "gvdb-builder.h"
-#include "gvdb-reader.h"
 
 #define EPHY_BOOKMARKS_FILE "bookmarks.gvdb"
 
@@ -52,77 +52,12 @@ enum {
 
 static guint       signals[LAST_SIGNAL];
 
-static void
-gvdb_hash_table_insert_variant (GHashTable *table,
-                                const char *key,
-                                GVariant   *value)
-{
-  GvdbItem *item;
-
-  item = gvdb_hash_table_insert (table, key);
-  gvdb_item_set_value (item, value);
-}
-
-static GVariant *
-build_variant (EphyBookmark *bookmark)
-{
-  GVariantBuilder builder;
-  GSequence *tags;
-  GSequenceIter *iter;
-
-  g_variant_builder_init (&builder, G_VARIANT_TYPE ("(xssdbas)"));
-
-  g_variant_builder_add (&builder, "x", ephy_bookmark_get_time_added (bookmark));
-  g_variant_builder_add (&builder, "s", ephy_bookmark_get_title (bookmark));
-  g_variant_builder_add (&builder, "s", ephy_bookmark_get_id (bookmark));
-  g_variant_builder_add (&builder, "d", ephy_bookmark_get_modification_time (bookmark));
-  g_variant_builder_add (&builder, "b", ephy_bookmark_is_uploaded (bookmark));
-
-  g_variant_builder_open (&builder, G_VARIANT_TYPE ("as"));
-  tags = ephy_bookmark_get_tags (bookmark);
-  for (iter = g_sequence_get_begin_iter (tags);
-       !g_sequence_iter_is_end (iter);
-       iter = g_sequence_iter_next (iter)) {
-    g_variant_builder_add (&builder, "s", g_sequence_get (iter));
-  }
-  g_variant_builder_close (&builder);
-
-  return g_variant_builder_end (&builder);
-}
-
-static void
-add_bookmark_to_table (EphyBookmark *bookmark, GHashTable *table)
-{
-  gvdb_hash_table_insert_variant (table,
-                                  ephy_bookmark_get_url (bookmark),
-                                  build_variant (bookmark));
-}
-
-static void
-add_tag_to_table (const char *tag, GHashTable *table)
-{
-  gvdb_hash_table_insert (table, tag);
-}
-
 static void
 ephy_bookmarks_manager_save_to_file (EphyBookmarksManager *self, GTask *task)
 {
-  GHashTable *root_table;
-  GHashTable *table;
   gboolean result;
 
-  root_table = gvdb_hash_table_new (NULL, NULL);
-
-  table = gvdb_hash_table_new (root_table, "tags");
-  g_sequence_foreach (self->tags, (GFunc)add_tag_to_table, table);
-  g_hash_table_unref (table);
-
-  table = gvdb_hash_table_new (root_table, "bookmarks");
-  g_sequence_foreach (self->bookmarks, (GFunc)add_bookmark_to_table, table);
-  g_hash_table_unref (table);
-
-  result = gvdb_table_write_contents (root_table, self->gvdb_filename, FALSE, NULL);
-  g_hash_table_unref (root_table);
+  result = ephy_bookmarks_export (self, self->gvdb_filename, NULL);
 
   if (task)
     g_task_return_boolean (task, result);
@@ -560,78 +495,7 @@ ephy_bookmarks_manager_save_to_file_finish (EphyBookmarksManager *self,
 void
 ephy_bookmarks_manager_load_from_file (EphyBookmarksManager *self)
 {
-  GvdbTable *root_table;
-  GvdbTable *table;
-  GSequence *bookmarks;
-  char **list;
-  int length;
-  int i;
-
-  /* Create a new table to hold data stored in file. */
-  root_table = gvdb_table_new (self->gvdb_filename, TRUE, NULL);
-  g_assert (root_table);
-
-  /* Add tags to the bookmark manager's sequence. */
-  table = gvdb_table_get_table (root_table, "tags");
-  g_assert (table);
-
-  /* Iterate over all keys (url's) in the table. */
-  list = gvdb_table_get_names (table, &length);
-  for (i = 0; i < length; i++)
-    ephy_bookmarks_manager_create_tag (self, list[i]);
-  g_strfreev (list);
-  gvdb_table_free (table);
-
-  /* Get bookmarks table */
-  table = gvdb_table_get_table (root_table, "bookmarks");
-  g_assert (table);
-
-  bookmarks = g_sequence_new (g_object_unref);
-
-  /* Iterate over all keys (url's) in the table. */
-  list = gvdb_table_get_names (table, &length);
-  for (i = 0; i < length; i++) {
-    EphyBookmark *bookmark;
-    GVariant *value;
-    GVariantIter *iter;
-    GSequence *tags;
-    char *tag;
-    const char *title;
-    gint64 time_added;
-    char *id;
-    double modified;
-    gboolean uploaded;
-
-    /* Obtain the correspoding GVariant. */
-    value = gvdb_table_get_value (table, list[i]);
-
-    g_variant_get (value, "(x&s&sdbas)", &time_added, &title, &id, &modified, &uploaded, &iter);
-
-    /* Add all stored tags in a GSequence. */
-    tags = g_sequence_new (g_free);
-    while (g_variant_iter_next (iter, "s", &tag)) {
-      g_sequence_insert_sorted (tags, tag,
-                                (GCompareDataFunc)ephy_bookmark_tags_compare,
-                                NULL);
-    }
-    g_variant_iter_free (iter);
-
-    /* Create the new bookmark. */
-    bookmark = ephy_bookmark_new (list[i], title, tags);
-    ephy_bookmark_set_time_added (bookmark, time_added);
-    ephy_bookmark_set_id (bookmark, id);
-    ephy_bookmark_set_modification_time (bookmark, modified);
-    ephy_bookmark_set_is_uploaded (bookmark, uploaded);
-    g_sequence_prepend (bookmarks, bookmark);
-
-    g_variant_unref (value);
-  }
-  ephy_bookmarks_manager_add_bookmarks (self, bookmarks);
-
-  g_strfreev (list);
-  gvdb_table_free (table);
-  g_sequence_free (bookmarks);
-  gvdb_table_free (root_table);
+  ephy_bookmarks_import (self, self->gvdb_filename, NULL);
 }
 
 void
