@@ -24,6 +24,7 @@
 
 #include "ephy-bookmark.h"
 #include "ephy-bookmark-row.h"
+#include "ephy-bookmarks-list-model.h"
 #include "ephy-bookmarks-manager.h"
 #include "ephy-debug.h"
 #include "ephy-shell.h"
@@ -40,8 +41,9 @@ struct _EphyBookmarksPopover {
   GtkWidget             *tag_detail_back_button;
   GtkWidget             *tag_detail_label;
 
-  EphyBookmarksManager  *manager;
-  EphyWindow            *window;
+  EphyBookmarksManager   *manager;
+  EphyBookmarksListModel *list_model;
+  EphyWindow             *window;
 };
 
 G_DEFINE_TYPE (EphyBookmarksPopover, ephy_bookmarks_popover, GTK_TYPE_POPOVER)
@@ -154,6 +156,8 @@ create_bookmark_row (gpointer item,
   EphyBookmark *bookmark = EPHY_BOOKMARK (item);
   GtkWidget *row;
 
+  g_assert (!ephy_bookmark_is_smart (bookmark));
+
   row = ephy_bookmark_row_new (bookmark);
   g_object_set_data_full (G_OBJECT (row), "type",
                           g_strdup (EPHY_LIST_BOX_ROW_TYPE_BOOKMARK),
@@ -221,6 +225,9 @@ ephy_bookmarks_popover_bookmark_added_cb (EphyBookmarksPopover *self,
   g_assert (EPHY_IS_BOOKMARK (bookmark));
   g_assert (EPHY_IS_BOOKMARKS_MANAGER (manager));
 
+  if (ephy_bookmark_is_smart (bookmark))
+    return;
+
   if (g_sequence_is_empty (ephy_bookmark_get_tags (bookmark))) {
     row = create_bookmark_row (bookmark, self);
     gtk_container_add (GTK_CONTAINER (self->tags_list_box), row);
@@ -240,6 +247,9 @@ ephy_bookmarks_popover_bookmark_removed_cb (EphyBookmarksPopover *self,
   g_assert (EPHY_IS_BOOKMARK (bookmark));
   g_assert (EPHY_IS_BOOKMARKS_MANAGER (manager));
 
+  if (ephy_bookmark_is_smart (bookmark))
+    return;
+
   children = gtk_container_get_children (GTK_CONTAINER (self->tags_list_box));
   for (l = children; l != NULL; l = l->next) {
     const char *type;
@@ -358,6 +368,8 @@ ephy_bookmarks_popover_show_tag_detail (EphyBookmarksPopover *self,
     EphyBookmark *bookmark = g_sequence_get (iter);
     GtkWidget *row;
 
+    if (ephy_bookmark_is_smart (bookmark))
+      continue;
     row = create_bookmark_row (bookmark, self);
     gtk_container_add (GTK_CONTAINER (self->tag_detail_list_box), row);
   }
@@ -366,6 +378,8 @@ ephy_bookmarks_popover_show_tag_detail (EphyBookmarksPopover *self,
 
   gtk_stack_set_visible_child_name (GTK_STACK (self->toplevel_stack),
                                     "tag_detail");
+
+  g_sequence_free (bookmarks);
 }
 
 static void
@@ -400,6 +414,16 @@ ephy_bookmarks_popover_list_box_row_activated_cb (EphyBookmarksPopover   *self,
   }
 }
 
+static void
+ephy_bookmarks_popover_dispose (GObject *object)
+{
+  EphyBookmarksPopover *self = EPHY_BOOKMARKS_POPOVER (object);
+
+  g_clear_object (&self->list_model);
+
+  G_OBJECT_CLASS (ephy_bookmarks_popover_parent_class)->dispose (object);
+}
+
 static void
 ephy_bookmarks_popover_set_property (GObject      *object,
                                      guint         prop_id,
@@ -423,6 +447,7 @@ ephy_bookmarks_popover_class_init (EphyBookmarksPopoverClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
+  object_class->dispose = ephy_bookmarks_popover_dispose;
   object_class->set_property = ephy_bookmarks_popover_set_property;
 
   obj_properties[PROP_WINDOW] =
@@ -458,6 +483,7 @@ ephy_bookmarks_popover_init (EphyBookmarksPopover *self)
   gtk_widget_init_template (GTK_WIDGET (self));
 
   self->manager = ephy_shell_get_bookmarks_manager (ephy_shell_get_default ());
+  self->list_model = ephy_bookmarks_list_model_new (self->manager);
 
   group = g_simple_action_group_new ();
   g_action_map_add_action_entries (G_ACTION_MAP (group), entries,
@@ -467,7 +493,7 @@ ephy_bookmarks_popover_init (EphyBookmarksPopover *self)
   g_object_unref (group);
 
   gtk_list_box_bind_model (GTK_LIST_BOX (self->bookmarks_list_box),
-                           G_LIST_MODEL (self->manager),
+                           G_LIST_MODEL (self->list_model),
                            create_bookmark_row,
                            self, NULL);
 
@@ -496,6 +522,8 @@ ephy_bookmarks_popover_init (EphyBookmarksPopover *self)
     EphyBookmark *bookmark = g_sequence_get (iter);
     GtkWidget *bookmark_row;
 
+    if (ephy_bookmark_is_smart (bookmark))
+      continue;
     bookmark_row = create_bookmark_row (bookmark, self);
     gtk_widget_show_all (bookmark_row);
     gtk_container_add (GTK_CONTAINER (self->tags_list_box), bookmark_row);
