@@ -38,6 +38,8 @@
 #include "ephy-history-service.h"
 #include "ephy-location-entry.h"
 #include "ephy-middle-clickable-button.h"
+#include "ephy-prefs.h"
+#include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-title-box.h"
 #include "ephy-title-widget.h"
@@ -70,6 +72,7 @@ struct _EphyHeaderBar {
   GtkWidget *downloads_revealer;
   GtkWidget *downloads_button;
   GtkWidget *downloads_popover;
+  GtkWidget *zoom_level_button;
 
   guint navigation_buttons_menu_timeout;
 };
@@ -177,6 +180,17 @@ sync_chromes_visibility (EphyHeaderBar *header_bar)
   gtk_widget_set_visible (header_bar->new_tab_button, chrome & EPHY_WINDOW_CHROME_TABSBAR);
 }
 
+static void
+homepage_url_changed (GSettings  *settings,
+                      const char *key,
+                      GtkWidget  *button)
+{
+  const char *setting;
+
+  setting = g_settings_get_string (settings, key);
+  gtk_widget_set_visible (button, setting && setting[0]);
+}
+
 typedef enum {
   EPHY_NAVIGATION_HISTORY_DIRECTION_BACK,
   EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD
@@ -646,6 +660,23 @@ ephy_header_bar_constructed (GObject *object)
   gtk_widget_show (GTK_WIDGET (button));
   gtk_header_bar_pack_start (GTK_HEADER_BAR (header_bar), button);
 
+  /* Homepage */
+  button = gtk_button_new ();
+  gtk_button_set_image (GTK_BUTTON (button),
+                        gtk_image_new_from_icon_name ("go-home-symbolic", GTK_ICON_SIZE_BUTTON));
+  gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (button), "win.home");
+  gtk_header_bar_pack_start (GTK_HEADER_BAR (header_bar), button);
+
+  embed_shell = ephy_embed_shell_get_default ();
+  if (ephy_embed_shell_get_mode (embed_shell) != EPHY_EMBED_SHELL_MODE_APPLICATION) {
+    homepage_url_changed (EPHY_SETTINGS_MAIN, EPHY_PREFS_HOMEPAGE_URL, button);
+    g_signal_connect (EPHY_SETTINGS_MAIN,
+                      "changed::" EPHY_PREFS_HOMEPAGE_URL,
+                      G_CALLBACK (homepage_url_changed),
+                      button);
+  }
+
   /* New Tab */
   button = gtk_button_new ();
   header_bar->new_tab_button = button;
@@ -656,7 +687,6 @@ ephy_header_bar_constructed (GObject *object)
   gtk_header_bar_pack_start (GTK_HEADER_BAR (header_bar), button);
 
   /* Title widget (location entry or title box) */
-  embed_shell = ephy_embed_shell_get_default ();
   if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_APPLICATION)
     header_bar->title_widget = EPHY_TITLE_WIDGET (ephy_title_box_new ());
   else
@@ -686,7 +716,7 @@ ephy_header_bar_constructed (GObject *object)
   g_type_ensure (G_TYPE_THEMED_ICON);
   builder = gtk_builder_new_from_resource ("/org/gnome/epiphany/gtk/page-menu-popover.ui");
   page_menu_popover = GTK_WIDGET (gtk_builder_get_object (builder, "page-menu-popover"));
-  g_object_set_data (G_OBJECT (page_menu_popover), "zoom-level-button", gtk_builder_get_object (builder, "zoom-level"));
+  header_bar->zoom_level_button = GTK_WIDGET (gtk_builder_get_object (builder, "zoom-level"));
   gtk_menu_button_set_popover (GTK_MENU_BUTTON (button), page_menu_popover);
   g_object_unref (builder);
 
@@ -803,15 +833,9 @@ ephy_header_bar_get_title_widget (EphyHeaderBar *header_bar)
 }
 
 GtkWidget *
-ephy_header_bar_get_page_menu_button (EphyHeaderBar *header_bar)
+ephy_header_bar_get_zoom_level_button (EphyHeaderBar *header_bar)
 {
-  return header_bar->page_menu_button;
-}
-
-GtkWidget *
-ephy_header_bar_get_new_tab_button (EphyHeaderBar *header_bar)
-{
-  return header_bar->new_tab_button;
+  return header_bar->zoom_level_button;
 }
 
 EphyWindow *
