@@ -47,11 +47,17 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
+#ifdef ENABLE_SYNC
+#include "ephy-notification.h"
+#endif
+
 struct _EphyShell {
   EphyEmbedShell parent_instance;
 
   EphySession *session;
+#ifdef ENABLE_SYNC
   EphySyncService *sync_service;
+#endif
   GList *windows;
   GObject *lockdown;
   EphyBookmarksManager *bookmarks_manager;
@@ -163,6 +169,30 @@ reopen_closed_tab (GSimpleAction *action,
   window_cmd_reopen_closed_tab (NULL, NULL, NULL);
 }
 
+static void
+import_bookmarks (GSimpleAction *action,
+                  GVariant      *parameter,
+                  gpointer       user_data)
+{
+  GtkWindow *window;
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (ephy_shell));
+
+  window_cmd_import_bookmarks (NULL, NULL, EPHY_WINDOW (window));
+}
+
+static void
+export_bookmarks (GSimpleAction *action,
+                  GVariant      *parameter,
+                  gpointer       user_data)
+{
+  GtkWindow *window;
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (ephy_shell));
+
+  window_cmd_export_bookmarks (NULL, NULL, EPHY_WINDOW (window));
+}
+
 static void
 show_history (GSimpleAction *action,
               GVariant      *parameter,
@@ -234,6 +264,8 @@ quit_application (GSimpleAction *action,
 static GActionEntry app_entries[] = {
   { "new-window", new_window, NULL, NULL, NULL },
   { "new-incognito", new_incognito_window, NULL, NULL, NULL },
+  { "import-bookmarks", import_bookmarks, NULL, NULL, NULL },
+  { "export-bookmarks", export_bookmarks, NULL, NULL, NULL },
   { "history", show_history, NULL, NULL, NULL },
   { "preferences", show_preferences, NULL, NULL, NULL },
   { "shortcuts", show_shortcuts, NULL, NULL, NULL },
@@ -281,6 +313,29 @@ download_started_cb (WebKitWebContext *web_context,
   g_object_unref (ephy_download);
 }
 
+#ifdef ENABLE_SYNC
+static void
+sync_tokens_load_finished_cb (EphySyncService *service,
+                              GError          *error,
+                              gpointer         user_data)
+{
+  EphyNotification *notification;
+
+  g_assert (EPHY_IS_SYNC_SERVICE (service));
+
+  /* If the tokens were successfully loaded, start the periodical sync.
+   * Otherwise, notify the user to sign in again. */
+  if (error == NULL) {
+    ephy_sync_service_start_periodical_sync (service, TRUE);
+  } else {
+    notification = ephy_notification_new (error->message,
+                                          _("Please visit Preferences and sign in "
+                                            "again to continue the sync process."));
+    ephy_notification_show (notification);
+  }
+}
+#endif
+
 static void
 ephy_shell_startup (GApplication *application)
 {
@@ -303,8 +358,6 @@ ephy_shell_startup (GApplication *application)
 
   mode = ephy_embed_shell_get_mode (embed_shell);
   if (mode != EPHY_EMBED_SHELL_MODE_APPLICATION) {
-    EphySyncService *service;
-
     g_action_map_add_action_entries (G_ACTION_MAP (application),
                                      app_entries, G_N_ELEMENTS (app_entries),
                                      application);
@@ -321,10 +374,13 @@ ephy_shell_startup (GApplication *application)
                               G_BINDING_SYNC_CREATE);
     }
 
-    /* Start the periodical sync now. */
-    service = ephy_sync_service_new ();
-    ephy_sync_service_start_periodical_sync (service, TRUE);
-    ephy_shell->sync_service = service;
+#ifdef ENABLE_SYNC
+    /* Create the sync service. */
+    ephy_shell->sync_service = ephy_sync_service_new ();
+    g_signal_connect (ephy_shell->sync_service,
+                      "sync-tokens-load-finished",
+                      G_CALLBACK (sync_tokens_load_finished_cb), NULL);
+#endif
 
     builder = gtk_builder_new ();
     gtk_builder_add_from_resource (builder,
@@ -594,7 +650,9 @@ ephy_shell_dispose (GObject *object)
   g_clear_pointer (&shell->history_window, gtk_widget_destroy);
   g_clear_object (&shell->prefs_dialog);
   g_clear_object (&shell->network_monitor);
+#ifdef ENABLE_SYNC
   g_clear_object (&shell->sync_service);
+#endif
   g_clear_object (&shell->bookmarks_manager);
 
   g_slist_free_full (shell->open_uris_idle_ids, remove_open_uris_idle_cb);
@@ -741,6 +799,7 @@ ephy_shell_get_session (EphyShell *shell)
   return shell->session;
 }
 
+#ifdef ENABLE_SYNC
 /**
  * ephy_shell_get_sync_service:
  * @shell: the #EphyShell
@@ -756,6 +815,7 @@ ephy_shell_get_sync_service (EphyShell *shell)
 
   return shell->sync_service;
 }
+#endif
 
 /**
  * ephy_shell_get_bookmarks_manager:
@@ -1018,8 +1078,9 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
     if (data->flags & EPHY_NEW_TAB_JUMP && mode != EPHY_EMBED_SHELL_MODE_TEST)
       gtk_window_present_with_time (GTK_WINDOW (data->window), data->user_time);
   } else {
-    ephy_web_view_load_homepage (ephy_embed_get_web_view (embed));
-    ephy_window_activate_location (data->window);
+    ephy_web_view_load_new_tab_page (ephy_embed_get_web_view (embed));
+    if (data->flags & EPHY_NEW_TAB_JUMP)
+      ephy_window_activate_location (data->window);
   }
 
   /* Set address from the very beginning. Looks odd in app mode if it appears later on. */
