@@ -62,6 +62,10 @@ struct _PrefsDialog {
   GtkDialog parent_instance;
 
   /* general */
+  GtkWidget *new_tab_homepage_radiobutton;
+  GtkWidget *blank_homepage_radiobutton;
+  GtkWidget *custom_homepage_radiobutton;
+  GtkWidget *custom_homepage_entry;
   GtkWidget *download_button_hbox;
   GtkWidget *download_button_label;
   GtkWidget *automatic_downloads_checkbutton;
@@ -103,6 +107,7 @@ struct _PrefsDialog {
   GHashTable *iso_639_table;
   GHashTable *iso_3166_table;
 
+#ifdef ENABLE_SYNC
   /* sync */
   GtkWidget *sync_authenticate_box;
   GtkWidget *sync_sign_in_box;
@@ -114,9 +119,13 @@ struct _PrefsDialog {
   WebKitWebView *fxa_web_view;
   WebKitUserContentManager *fxa_manager;
   WebKitUserScript *fxa_script;
-  guint source_id;
+  guint fxa_id;
+#else
+  GtkWidget *notebook;
+#endif
 };
 
+#ifdef ENABLE_SYNC
 typedef struct {
   PrefsDialog *dialog;
   char        *email;
@@ -129,6 +138,7 @@ typedef struct {
   guint8      *respHMACkey;
   guint8      *respXORkey;
 } FxACallbackData;
+#endif
 
 enum {
   SEARCH_ENGINE_COL_NAME,
@@ -139,6 +149,7 @@ enum {
 
 G_DEFINE_TYPE (PrefsDialog, prefs_dialog, GTK_TYPE_DIALOG)
 
+#ifdef ENABLE_SYNC
 static FxACallbackData *
 fxa_callback_data_new (PrefsDialog *dialog,
                        const char  *email,
@@ -189,6 +200,7 @@ fxa_callback_data_free (FxACallbackData *data)
 
   g_slice_free (FxACallbackData, data);
 }
+#endif
 
 static void
 prefs_dialog_finalize (GObject *object)
@@ -206,6 +218,7 @@ prefs_dialog_finalize (GObject *object)
   g_hash_table_destroy (dialog->iso_639_table);
   g_hash_table_destroy (dialog->iso_3166_table);
 
+#ifdef ENABLE_SYNC
   if (dialog->fxa_web_view != NULL) {
     webkit_user_content_manager_unregister_script_message_handler (dialog->fxa_manager,
                                                                    "accountsCommandHandler");
@@ -213,14 +226,16 @@ prefs_dialog_finalize (GObject *object)
     g_object_unref (dialog->fxa_manager);
   }
 
-  if (dialog->source_id != 0) {
-    g_source_remove (dialog->source_id);
-    dialog->source_id = 0;
+  if (dialog->fxa_id != 0) {
+    g_source_remove (dialog->fxa_id);
+    dialog->fxa_id = 0;
   }
+#endif
 
   G_OBJECT_CLASS (prefs_dialog_parent_class)->finalize (object);
 }
 
+#ifdef ENABLE_SYNC
 static void
 hide_fxa_iframe (PrefsDialog *dialog,
                  const char  *email)
@@ -243,6 +258,41 @@ hide_fxa_iframe (PrefsDialog *dialog,
   g_free (account);
 }
 
+static void
+sync_tokens_store_finished_cb (EphySyncService *service,
+                               GError          *error,
+                               PrefsDialog     *dialog)
+{
+  g_assert (EPHY_IS_SYNC_SERVICE (service));
+  g_assert (EPHY_IS_PREFS_DIALOG (dialog));
+
+  if (error == NULL) {
+    /* Show the 'Signed in' panel. */
+    hide_fxa_iframe (dialog, ephy_sync_service_get_user_email (service));
+
+    /* Do a first time sync and set a periodical sync to be executed. */
+    ephy_sync_service_sync_bookmarks (service, TRUE);
+    ephy_sync_service_start_periodical_sync (service, FALSE);
+  } else {
+    char *message;
+
+    /* Destroy the current session. */
+    ephy_sync_service_destroy_session (service, NULL);
+
+    /* Unset the email and tokens. */
+    g_settings_set_string (EPHY_SETTINGS_MAIN, EPHY_PREFS_SYNC_USER, "");
+    ephy_sync_service_clear_tokens (service);
+
+    /* Display the error message to the user. */
+    message = g_strdup_printf ("<span fgcolor='#e6780b'>%s</span>", error->message);
+    gtk_label_set_markup (GTK_LABEL (dialog->sync_sign_in_details), message);
+    gtk_widget_set_visible (dialog->sync_sign_in_details, TRUE);
+    webkit_web_view_load_uri (dialog->fxa_web_view, FXA_IFRAME_URL);
+
+    g_free (message);
+  }
+}
+
 static gboolean
 poll_fxa_server (gpointer user_data)
 {
@@ -259,10 +309,10 @@ poll_fxa_server (gpointer user_data)
                                       data->sessionToken, data->keyFetchToken,
                                       data->unwrapBKey, bundle,
                                       data->respHMACkey, data->respXORkey);
-    hide_fxa_iframe (data->dialog, data->email);
 
+    g_free (bundle);
     fxa_callback_data_free (data);
-    data->dialog->source_id = 0;
+    data->dialog->fxa_id = 0;
 
     return G_SOURCE_REMOVE;
   }
@@ -363,6 +413,11 @@ server_message_received_cb (WebKitUserContentManager *manager,
       goto out;
     }
 
+    /* Derive tokenID, reqHMACkey, respHMACkey and respXORkey from the keyFetchToken.
+     * tokenID and reqHMACkey are used to make a HAWK request to the "GET /account/keys"
+     * API. The server looks up the stored table entry with tokenID, checks the request
+     * HMAC for validity, then returns the pre-encrypted response.
+     * See https://github.com/mozilla/fxa-auth-server/wiki/onepw-protocol#fetching-sync-keys */
     ephy_sync_crypto_process_key_fetch_token (keyFetchToken,
                                               &tokenID, &reqHMACkey,
                                               &respHMACkey, &respXORkey);
@@ -380,7 +435,7 @@ server_message_received_cb (WebKitUserContentManager *manager,
       cb_data = fxa_callback_data_new (dialog, email, uid, sessionToken,
                                        keyFetchToken, unwrapBKey, tokenID,
                                        reqHMACkey, respHMACkey, respXORkey);
-      dialog->source_id = g_timeout_add_seconds (2, (GSourceFunc)poll_fxa_server, cb_data);
+      dialog->fxa_id = g_timeout_add_seconds (2, (GSourceFunc)poll_fxa_server, cb_data);
 
       g_free (text);
     } else {
@@ -389,7 +444,6 @@ server_message_received_cb (WebKitUserContentManager *manager,
       bundle = ephy_sync_service_start_sign_in (service, tokenID, reqHMACkey);
       ephy_sync_service_finish_sign_in (service, email, uid, sessionToken, keyFetchToken,
                                         unwrapBKey, bundle, respHMACkey, respXORkey);
-      hide_fxa_iframe (dialog, email);
 
       g_free (bundle);
     }
@@ -409,7 +463,6 @@ out:
 static void
 setup_fxa_sign_in_view (PrefsDialog *dialog)
 {
-  EphyEmbedShell *shell;
   const char *js = "\"use strict\";"
                    "function handleAccountsCommand(evt) {"
                    "  let j = {type: evt.type, detail: evt.detail};"
@@ -417,8 +470,6 @@ setup_fxa_sign_in_view (PrefsDialog *dialog)
                    "};"
                    "window.addEventListener(\"FirefoxAccountsCommand\", handleAccountsCommand);";
 
-  shell = ephy_embed_shell_get_default ();
-
   dialog->fxa_script = webkit_user_script_new (js,
                                                WEBKIT_USER_CONTENT_INJECT_TOP_FRAME,
                                                WEBKIT_USER_SCRIPT_INJECT_AT_DOCUMENT_END,
@@ -433,7 +484,6 @@ setup_fxa_sign_in_view (PrefsDialog *dialog)
                                                                "accountsCommandHandler");
 
   dialog->fxa_web_view = WEBKIT_WEB_VIEW (g_object_new (WEBKIT_TYPE_WEB_VIEW,
-                                                        "web-context", ephy_embed_shell_get_web_context (shell),
                                                         "user-content-manager", dialog->fxa_manager,
                                                         "settings", ephy_embed_prefs_get_settings (),
                                                         NULL));
@@ -479,6 +529,7 @@ on_sync_sign_out_button_clicked (GtkWidget   *button,
                       TRUE, TRUE, 0);
   gtk_widget_set_visible (dialog->sync_sign_in_details, FALSE);
 }
+#endif
 
 static void
 on_manage_cookies_button_clicked (GtkWidget   *button,
@@ -517,6 +568,10 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/epiphany/prefs-dialog.ui");
   /* general */
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, new_tab_homepage_radiobutton);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, blank_homepage_radiobutton);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, custom_homepage_radiobutton);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, custom_homepage_entry);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, automatic_downloads_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, search_engine_combo);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, restore_session_checkbutton);
@@ -551,6 +606,7 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, lang_down_button);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, enable_spell_checking_checkbutton);
 
+#ifdef ENABLE_SYNC
   /* sync */
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, sync_authenticate_box);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, sync_sign_in_box);
@@ -558,10 +614,12 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, sync_sign_out_box);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, sync_sign_out_details);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, sync_sign_out_button);
+#else
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, notebook);
+#endif
 
   gtk_widget_class_bind_template_callback (widget_class, on_manage_cookies_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_manage_passwords_button_clicked);
-  gtk_widget_class_bind_template_callback (widget_class, on_sync_sign_out_button_clicked);
 }
 
 static void
@@ -1369,6 +1427,106 @@ create_search_engine_combo (GtkComboBox *combo)
                                 NULL);
 }
 
+static gboolean
+new_tab_homepage_get_mapping (GValue   *value,
+                              GVariant *variant,
+                              gpointer  user_data)
+{
+  const char *setting;
+
+  setting = g_variant_get_string (variant, NULL);
+  if (!setting || setting[0] == '\0')
+    g_value_set_boolean (value, TRUE);
+
+  return TRUE;
+}
+
+static GVariant *
+new_tab_homepage_set_mapping (const GValue       *value,
+                              const GVariantType *expected_type,
+                              gpointer            user_data)
+{
+  if (!g_value_get_boolean (value))
+    return NULL;
+  return g_variant_new_string ("");
+}
+
+static gboolean
+blank_homepage_get_mapping (GValue   *value,
+                            GVariant *variant,
+                            gpointer  user_data)
+{
+  const char *setting;
+
+  setting = g_variant_get_string (variant, NULL);
+  if (g_strcmp0 (setting, "about:blank") == 0)
+      g_value_set_boolean (value, TRUE);
+
+  return TRUE;
+}
+
+static GVariant *
+blank_homepage_set_mapping (const GValue       *value,
+                            const GVariantType *expected_type,
+                            gpointer            user_data)
+{
+  if (!g_value_get_boolean (value))
+    return NULL;
+  return g_variant_new_string ("about:blank");
+}
+
+static gboolean
+custom_homepage_get_mapping (GValue   *value,
+                             GVariant *variant,
+                             gpointer  user_data)
+{
+  const char *setting;
+
+  setting = g_variant_get_string (variant, NULL);
+  if (setting && setting[0] != '\0' && g_strcmp0 (setting, "about:blank") != 0)
+    g_value_set_boolean (value, TRUE);
+  return TRUE;
+}
+
+static GVariant *
+custom_homepage_set_mapping (const GValue       *value,
+                             const GVariantType *expected_type,
+                             gpointer            user_data)
+{
+  PrefsDialog *dialog = user_data;
+  const char *setting;
+
+  if (!g_value_get_boolean (value)) {
+    gtk_widget_set_sensitive (dialog->custom_homepage_entry, FALSE);
+    return NULL;
+  }
+
+  gtk_widget_set_sensitive (dialog->custom_homepage_entry, TRUE);
+  gtk_widget_grab_focus (dialog->custom_homepage_entry);
+  setting = gtk_entry_get_text (GTK_ENTRY (dialog->custom_homepage_entry));
+  if (!setting || setting[0] == '\0')
+    return NULL;
+
+  gtk_entry_set_text (GTK_ENTRY (dialog->custom_homepage_entry), setting);
+  return g_variant_new_string (setting);
+}
+
+static void
+custom_homepage_entry_changed (GtkEntry    *entry,
+                               PrefsDialog *dialog)
+{
+  g_settings_set_string (EPHY_SETTINGS_MAIN, EPHY_PREFS_HOMEPAGE_URL,
+                         gtk_entry_get_text (entry));
+}
+
+static void
+custom_homepage_entry_icon_released (GtkEntry            *entry,
+                                     GtkEntryIconPosition icon_pos)
+{
+  if (icon_pos == GTK_ENTRY_ICON_SECONDARY)
+    gtk_entry_set_text (entry, "");
+}
+
 static gboolean
 restore_session_get_mapping (GValue   *value,
                              GVariant *variant,
@@ -1400,6 +1558,49 @@ setup_general_page (PrefsDialog *dialog)
   settings = ephy_settings_get (EPHY_PREFS_SCHEMA);
   web_settings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
 
+  g_settings_bind_with_mapping (settings,
+                                EPHY_PREFS_HOMEPAGE_URL,
+                                dialog->new_tab_homepage_radiobutton,
+                                "active",
+                                G_SETTINGS_BIND_DEFAULT,
+                                new_tab_homepage_get_mapping,
+                                new_tab_homepage_set_mapping,
+                                NULL,
+                                NULL);
+  g_settings_bind_with_mapping (settings,
+                                EPHY_PREFS_HOMEPAGE_URL,
+                                dialog->blank_homepage_radiobutton,
+                                "active",
+                                G_SETTINGS_BIND_DEFAULT,
+                                blank_homepage_get_mapping,
+                                blank_homepage_set_mapping,
+                                NULL,
+                                NULL);
+  g_settings_bind_with_mapping (settings,
+                                EPHY_PREFS_HOMEPAGE_URL,
+                                dialog->custom_homepage_radiobutton,
+                                "active",
+                                G_SETTINGS_BIND_DEFAULT,
+                                custom_homepage_get_mapping,
+                                custom_homepage_set_mapping,
+                                dialog,
+                                NULL);
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->custom_homepage_radiobutton))) {
+    gtk_widget_set_sensitive (dialog->custom_homepage_entry, TRUE);
+    gtk_entry_set_text (GTK_ENTRY (dialog->custom_homepage_entry),
+                        g_settings_get_string (EPHY_SETTINGS_MAIN, EPHY_PREFS_HOMEPAGE_URL));
+  } else {
+    gtk_widget_set_sensitive (dialog->custom_homepage_entry, FALSE);
+    gtk_entry_set_text (GTK_ENTRY (dialog->custom_homepage_entry), "");
+  }
+
+  g_signal_connect (dialog->custom_homepage_entry, "changed",
+                    G_CALLBACK (custom_homepage_entry_changed),
+                    dialog);
+  g_signal_connect (dialog->custom_homepage_entry, "icon-release",
+                    G_CALLBACK (custom_homepage_entry_icon_released),
+                    NULL);
+
   g_settings_bind (settings,
                    EPHY_PREFS_AUTO_DOWNLOADS,
                    dialog->automatic_downloads_checkbutton,
@@ -1551,6 +1752,7 @@ setup_language_page (PrefsDialog *dialog)
   create_language_section (dialog);
 }
 
+#ifdef ENABLE_SYNC
 static void
 setup_sync_page (PrefsDialog *dialog)
 {
@@ -1576,7 +1778,12 @@ setup_sync_page (PrefsDialog *dialog)
     g_free (text);
     g_free (account);
   }
+
+  g_signal_connect_object (service, "sync-tokens-store-finished",
+                           G_CALLBACK (sync_tokens_store_finished_cb),
+                           dialog, 0);
 }
+#endif
 
 static void
 prefs_dialog_init (PrefsDialog *dialog)
@@ -1587,8 +1794,16 @@ prefs_dialog_init (PrefsDialog *dialog)
   setup_fonts_page (dialog);
   setup_privacy_page (dialog);
   setup_language_page (dialog);
+#ifdef ENABLE_SYNC
   setup_sync_page (dialog);
 
+  /* TODO: Switch back to using a template callback in class_init once sync is unconditionally enabled. */
+  g_signal_connect (dialog->sync_sign_out_button, "clicked",
+                    G_CALLBACK (on_sync_sign_out_button_clicked), dialog);
+#else
+  gtk_notebook_remove_page (GTK_NOTEBOOK (dialog->notebook), -1);
+#endif
+
   ephy_gui_ensure_window_group (GTK_WINDOW (dialog));
   g_signal_connect (dialog, "response",
                     G_CALLBACK (prefs_dialog_response_cb), dialog);
